#ifndef _OWLPS_POSITIONING_CALIBRATIONREQUEST_HH_
#define _OWLPS_POSITIONING_CALIBRATIONREQUEST_HH_

class ReferencePoint ;

#include "request.hh"
#include "direction.hh"

/// Represents a calibration Request sent by a mobile
class CalibrationRequest: public Request
{
protected:
  /// The reference point the mobile calibrates
  ReferencePoint *reference_point ;
  /// The direction in which the mobile was during the measurement
  Direction direction ;

public:
  CalibrationRequest(void):
    reference_point(NULL), direction(Direction()) {}

  CalibrationRequest(const CalibrationRequest &source):
    Request(source), reference_point(source.reference_point),
    direction(source.direction) {}

  CalibrationRequest(const Request &source,
                     ReferencePoint *_reference_point = NULL,
                     const Direction &_direction = Direction()):
    Request(source), reference_point(_reference_point),
    direction(_direction) {}

  ~CalibrationRequest(void) {}

  /** @name Read accessors */
  //@{
  const Direction& get_direction(void) const ;
  ReferencePoint* get_reference_point(void) const ;
  //@}

  /** @name Write accessors */
  //@{
  void set_direction(const Direction &_direction) ;
  void set_reference_point(const ReferencePoint *_rp) ;
  /// Add the CalibrationRequest to the #reference_point list of requests
  void reference_point_backward_link(void) const ;
  void clear(void) ;
  //@}

  /** @name Operators */
  //@{
  const CalibrationRequest& operator=(const CalibrationRequest &source) ;
  bool operator==(const CalibrationRequest &source) const ;
  bool operator!=(const CalibrationRequest &source) const ;
  //@}

  /// Hash a CalibrationRequest
  friend std::size_t hash_value(const CalibrationRequest &source) ;
} ;



/* *** Read accessors *** */


inline ReferencePoint* CalibrationRequest::get_reference_point(void) const
{
  return reference_point ;
}


inline const Direction& CalibrationRequest::get_direction(void) const
{
  return direction ;
}



/* *** Write accessors *** */


inline void CalibrationRequest::set_reference_point(
  const ReferencePoint *_rp)
{
  reference_point = const_cast<ReferencePoint*>(_rp) ;
}


inline void CalibrationRequest::
set_direction(const Direction &_direction)
{
  direction = _direction ;
}



/* *** Operators *** */


inline bool CalibrationRequest::
operator!=(const CalibrationRequest &source) const
{
  return !(*this == source) ;
}



#endif // _OWLPS_POSITIONING_CALIBRATIONREQUEST_HH_
