#include "input.hh"
#include "inputcsv.hh"
#include "inputlogcsv.hh"
#include "calibrationrequest.hh"
#include "configuration.hh"
#include "posexcept.hh"
#include "stock.hh"

#include <string>

using namespace std ;



/* *** Constructors *** */


Input::Input():
  medium(NULL)
{
  initialise_input_medium() ;
  initialise_log_media() ;
}


Input::~Input()
{
  delete medium ;

  for (vector<InputLogMedium*>::const_iterator i = log_media.begin() ;
       i != log_media.end() ; ++i)
    delete *i ;
  log_media.clear() ;
}



/* *** Operations *** */


void Input::initialise_input_medium()
{
  if (! Configuration::is_configured("input.medium"))
    throw missing_configuration(
      "No input medium specified in configuration!") ;

  const string &medium_name =
    Configuration::string_value("input.medium") ;

  if (medium_name == "CSV")
    {
      if (! Configuration::is_configured("input.csv-file"))
        throw missing_configuration(
          "No input CSV file specified in the configuration!") ;
      medium = new
      InputCSV(Configuration::string_value("input.csv-file")) ;
    }

  else
    throw bad_configuration(
      "The specified input medium « "+ medium_name +" » is unknown!") ;
}


void Input::initialise_log_media()
{
  if (! Configuration::is_configured("log.medium"))
    return ;

  if (Configuration::value_exists_in_string_vector("log.medium", "none"))
    return ;

  const vector<string> &media_names =
    Configuration::string_vector_value("log.medium") ;

  for (vector<string>::const_iterator i = media_names.begin() ;
       i != media_names.end() ; ++i)
    {
      if (*i == "CSV")
        initialise_log_csv() ;

      else
        throw bad_configuration(
          "The input medium « "+ *i +
          " » specified for request logging is unknown!") ;
    }
}


void Input::initialise_log_csv()
{
  if (! Configuration::is_configured("log.csv-file"))
    throw missing_configuration(
      "No log CSV file specified in the configuration!") ;

  log_media.push_back(
    new InputLogCSV(
      Configuration::string_value("log.csv-file"))) ;
}


const Request& Input::get_next_request() const
{
  if (! eof())
    {
      medium->get_next_request() ;
      log_current_request() ;
    }

  return medium->get_current_request() ;
}


bool Input::eof() const
{
  if (medium == NULL)
    throw null_input_medium() ;
  return medium->eof() ;
}


void Input::log_current_request() const
{
  for (vector<InputLogMedium*>::const_iterator i = log_media.begin() ;
       i != log_media.end() ; ++i)
    (*i)->log_request(medium->get_current_request()) ;
}
